package jp.ac.gifunct.ktajima.llservey;

import android.app.Activity;
import android.content.Intent;
import android.graphics.Bitmap;
import android.hardware.Camera;
import android.media.MediaScannerConnection;
import android.os.Bundle;
import android.os.Environment;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.widget.Button;
import android.widget.Toast;

import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.text.SimpleDateFormat;
import java.util.GregorianCalendar;
import java.util.List;

public class CameraActivity extends AppCompatActivity {
    private String surveyName;
    private String targetName;

    private SurfaceView mySurfaceView;
    private Camera myCamera; //hardware
    private Button takephotoButton;
    File lastPicture = null;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_camera);
        //SurfaceView
        mySurfaceView = (SurfaceView)findViewById(R.id.surfaceView);

        //SurfaceHolder(SVの制御に使うInterface）
        SurfaceHolder holder = mySurfaceView.getHolder();
        //コールバックを設定
        holder.addCallback(callback);

        //タッチイベントを設定
        mySurfaceView.setOnTouchListener(touchListener);

        this.takephotoButton = (Button)findViewById(R.id.button_take);
        this.takephotoButton.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                CameraActivity.this.PhotoButtonClicked(v);
            }
        });

        Intent intent = getIntent();
        if(intent != null){
            this.surveyName = intent.getStringExtra(ToolsUtility.IntentKey_SurveyID);
            this.targetName = intent.getStringExtra(ToolsUtility.IntentKey_PictureTargetID);
        }

    }

    private void PhotoButtonClicked(View v) {
        if (this.myCamera != null) {
            // 撮影実行
            this.myCamera.cancelAutoFocus();
            this.myCamera.takePicture(shutterListener_, null, pictureListener_);
        }
    }
    // シャッターが押されたときに呼ばれるコールバック
    private Camera.ShutterCallback shutterListener_ = new Camera.ShutterCallback() {
        public void onShutter() {
        }
    };

    // JPEGイメージ生成後に呼ばれるコールバック
    private Camera.PictureCallback pictureListener_ = new Camera.PictureCallback() {
        // データ生成完了
        public void onPictureTaken(byte[] data, Camera camera) {
            // SDカードにJPEGデータを保存する
            if (data != null) {


                FileOutputStream fos = null;
                try {
                    File outputFile = ToolsUtility.makePictureFileName(CameraActivity.this.surveyName);
                    //Log.i("outputFile", outputFile.getPath());
                    fos = new FileOutputStream(outputFile);
                    Bitmap image = ToolsUtility.tilt(ToolsUtility.makeSquare(ToolsUtility.openImageFromByteArray(data)), 90);
                    image.compress(Bitmap.CompressFormat.JPEG, 100, fos);
                    fos.flush();
                    fos.close();
                    CameraActivity.this.lastPicture = outputFile;


                    String[] paths = {outputFile.getPath()};
                    String[] mimeTypes = {"image/jpeg"};
                    MediaScannerConnection.scanFile(getApplicationContext(),
                            paths,
                            mimeTypes,
                            null);
                    CameraActivity.this.onBackPressed();


                } catch (Exception e) {
                    e.printStackTrace();
                }

                // プレビューを再開する
                //camera.startPreview();
            }
        }
    };

    //タッチイベント
    private View.OnTouchListener touchListener = new View.OnTouchListener(){
        public boolean onTouch(View v, MotionEvent event) {
            if( event.getAction() == MotionEvent.ACTION_DOWN ) {

                //中心に合わせる
                //List<Camera.Area> area = new ArrayList<Camera.Area>();
                //area.add(new Camera.Area(new Rect(0, 0, 0, 0), 1));
                //Camera.Parameters pr = myCamera.getParameters();
                //pr.setFocusAreas(area);
                //myCamera.setParameters(pr);
                myCamera.cancelAutoFocus();
                myCamera.autoFocus(null);
            }
            return false;
        }
    };

    //コールバック
    private SurfaceHolder.Callback callback = new SurfaceHolder.Callback() {
        @Override
        public void surfaceCreated(SurfaceHolder surfaceHolder) {

            //CameraOpen
            myCamera = Camera.open();

            //出力をSurfaceViewに設定
            try{
                myCamera.setPreviewDisplay(surfaceHolder);
                Camera.Parameters pr = myCamera.getParameters();
                Camera.Size size = pr.getPictureSize();
                Log.i("Size", "getPictureSize width:" + size.width + " size.height:" + size.height);
                List<Camera.Size> sizes = pr.getSupportedPictureSizes();
                Camera.Size picturesize = null;
                for(int i=0;i<sizes.size();i++) {
                //    Log.i("Size", "supportSize"+i+" width:" + sizes.get(i).width + " size.height:" + sizes.get(i).height);
                    if(sizes.get(i).width < 1000){
                        if(i > 0) i--;
                        picturesize = sizes.get(i);
                        break;
                    }
                }
                //List<Camera.Size> ss = pr.getSupportedPictureSizes();
                //Camera.Size s = ss.get(0);
                pr.setPictureSize(picturesize.width, picturesize.height);
                myCamera.setParameters(pr);
            }catch(Exception e){
                e.printStackTrace();
            }

        }

        @Override
        public void surfaceChanged(SurfaceHolder surfaceHolder,  int format, int width, int height) {
            try{
                int d = 90;
                myCamera.setDisplayOrientation(d);
                myCamera.setPreviewDisplay(surfaceHolder);
                Camera.Parameters pr = myCamera.getParameters();
                //pr.setPreviewSize(640, 640);
                myCamera.setParameters(pr);
                myCamera.startPreview();
            }catch(Exception e){
                e.printStackTrace();
            }
        }

        @Override
        public void surfaceDestroyed(SurfaceHolder surfaceHolder) {

            //片付け
            myCamera.release();
            myCamera = null;
        }
    };

    @Override
    public void onBackPressed(){
        // intentの作成
        Intent intent = new Intent();
        // intentへ添え字付で値を保持させる
        intent.putExtra("lastpic",this.lastPicture.getPath());
        // 返却したい結果ステータスをセットする
        setResult( Activity.RESULT_OK, intent );
        // アクティビティを終了させる
        this.finish();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_shooting, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            return true;
        }

        return super.onOptionsItemSelected(item);
    }
}
