package jp.ac.gifunct.ktajima.llservey;

import android.app.Activity;
import android.content.Intent;
import android.support.v4.app.FragmentActivity;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.Toast;

import com.google.android.gms.maps.CameraUpdate;
import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.CameraPosition;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;

public class MapsActivity extends FragmentActivity {

    private GoogleMap mMap; // Might be null if Google Play services APK is not available.
    private double Latitude;
    private double Longitude;
    private double Altitude;
    private String placeType;

    private Button pinEditButton;

    private LatLng savePoint;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_maps);
        setUpMapIfNeeded();

        Intent intent = getIntent();
        if(intent != null){
            this.placeType = intent.getStringExtra("place");
            this.Latitude = intent.getDoubleExtra("lat",0);
            this.Longitude = intent.getDoubleExtra("long",0);
            this.Altitude = intent.getDoubleExtra("alt",0);
        }

        this.pinEditButton = (Button)findViewById(R.id.button_mapedit);
        this.pinEditButton.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                MapsActivity.this.pinEditButtonClicked(v);
            }
        });

    }



    private void pinEditButtonClicked(View v) {
        Toast ts = Toast.makeText(this, "Tap the pictured place", Toast.LENGTH_LONG);
        this.editEnabled = true;
        ts.show();

    }

    @Override
    protected void onResume() {
        super.onResume();
        setUpMapIfNeeded();
        this.savePoint = new LatLng(this.Latitude, this.Longitude);
        if(this.Latitude == 0 && this.Longitude == 0){
            CameraPosition cpos = new CameraPosition(this.savePoint, 0, 0, 0);
            mMap.moveCamera(CameraUpdateFactory.newCameraPosition(cpos));
        } else {
            mMap.addMarker(new MarkerOptions().position(this.savePoint).title(this.placeType));
            CameraPosition cpos = new CameraPosition(this.savePoint, 15, 0, 0);
            mMap.moveCamera(CameraUpdateFactory.newCameraPosition(cpos));
        }

        mMap.setOnMapClickListener(mapClickListener);
    }

    protected boolean editEnabled = false;
    protected Marker editMaker = null;
    protected GoogleMap.OnMapClickListener mapClickListener = new  GoogleMap.OnMapClickListener() {
        @Override
        public void onMapClick(LatLng latLng) {
            if(editEnabled == false ) return;
            if(editMaker != null){
                editMaker.remove();
            }
            editMaker = mMap.addMarker(new MarkerOptions().position(latLng).title("NewPlace").icon(BitmapDescriptorFactory.defaultMarker(BitmapDescriptorFactory.HUE_GREEN)));
            savePoint = latLng;
            editEnabled = false;
        }
    };
    /**
     * Sets up the map if it is possible to do so (i.e., the Google Play services APK is correctly
     * installed) and the map has not already been instantiated.. This will ensure that we only ever
     * call {@link #setUpMap()} once when {@link #mMap} is not null.
     * <p/>
     * If it isn't installed {@link SupportMapFragment} (and
     * {@link com.google.android.gms.maps.MapView MapView}) will show a prompt for the user to
     * install/update the Google Play services APK on their device.
     * <p/>
     * A user can return to this FragmentActivity after following the prompt and correctly
     * installing/updating/enabling the Google Play services. Since the FragmentActivity may not
     * have been completely destroyed during this process (it is likely that it would only be
     * stopped or paused), {@link #onCreate(Bundle)} may not be called again so we should call this
     * method in {@link #onResume()} to guarantee that it will be called.
     */
    private void setUpMapIfNeeded() {
        // Do a null check to confirm that we have not already instantiated the map.
        if (mMap == null) {
            // Try to obtain the map from the SupportMapFragment.
            mMap = ((SupportMapFragment) getSupportFragmentManager().findFragmentById(R.id.map))
                    .getMap();
            // Check if we were successful in obtaining the map.
            if (mMap != null) {
                setUpMap();
            }
        }
    }

    /**
     * This is where we can add markers or lines, add listeners or move the camera. In this case, we
     * just add a marker near Africa.
     * <p/>
     * This should only be called once and when we are sure that {@link #mMap} is not null.
     */
    private void setUpMap() {
        //mMap.addMarker(new MarkerOptions().position(new LatLng(0, 0)).title("Marker"));
    }

    @Override
    public void onBackPressed(){
        // intentの作成
        Intent intent = new Intent();
        // intentへ添え字付で値を保持させる
        intent.putExtra("lat",this.savePoint.latitude);
        intent.putExtra("lon",this.savePoint.longitude);
        // 返却したい結果ステータスをセットする
        setResult( Activity.RESULT_OK, intent );
        // アクティビティを終了させる
        this.finish();
    }
}
