package jp.ac.gifunct.ktajima.llservey;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import java.util.ArrayList;

public class SurveySettingActivity extends AppCompatActivity {

    private EditText userNameLabel;
    private EditText surveyNameLabel;
    private ListView listView;
    private ArrayAdapter<String> surveyListAdapter;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_survey_setting);

        //GUI設定
        userNameLabel = (EditText)findViewById(R.id.editText_observer);
        surveyNameLabel = (EditText)findViewById(R.id.surveyName);
        surveyNameLabel.setFocusable(false);

        //値の引き渡し
        Intent intent = getIntent();
        if(intent != null) {
            if (intent.hasExtra("observer")) {
                userNameLabel.setText(intent.getStringExtra("observer"));
            }
            if (intent.hasExtra("surveyname")) {
                surveyNameLabel.setText(intent.getStringExtra("surveyname"));
            }
        }

        //調査ID一覧の取得
        final ArrayList<String> surveys = new ArrayList<String>();
        surveys.addAll(ToolsUtility.dataSetting.keySet());
        //リストにセット
        this.listView = (ListView)findViewById(R.id.listView_survey);
        this.surveyListAdapter = new ArrayAdapter(this, R.layout.textlist);
        for(final String surveyName:surveys) {
            surveyListAdapter.add(surveyName);
        }
        this.listView.setAdapter(surveyListAdapter);
        this.listView.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                String sname = surveys.get(position);
                Log.i("sname", sname);
                listElementClicked(sname);
            }
        });
        ((Button)findViewById(R.id.button_newservey)).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                SurveySettingActivity.this.newSurveyButtonClicked(v);
            }
        });
    }

    private void newSurveyButtonClicked(View v) {
        this.openAddTextDialog("Input new Survey Name", new DialogInterface.OnDismissListener() {
            @Override
            public void onDismiss(DialogInterface dialog) {
                SurveySettingActivity.this.newSurveyNameFixed();
            }
        });
    }

    private void newSurveyNameFixed(){
        if(addTextDialogString == null){
            return;
        }
        //新規調査の追加
        this.surveyListAdapter.add(addTextDialogString);
        surveyNameLabel.setText(addTextDialogString);
        ToolsUtility.addNewSurvey(addTextDialogString);
    }

    private String addTextDialogString;
    private void openAddTextDialog(String title,DialogInterface.OnDismissListener listner) {
        addTextDialogString = null;
        final EditText editView = new EditText(this);
        AlertDialog.Builder builder = new AlertDialog.Builder(this)
                .setIcon(android.R.drawable.ic_dialog_info)
                .setTitle(title)
                .setView(editView)
                .setPositiveButton("OK", new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int whichButton) {
                        addTextDialogString = editView.getText().toString();
                    }
                })
                .setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int whichButton) {
                    }
                });
        Dialog dialog = builder.show();

        dialog.setOnDismissListener(listner);
    }

    private void listElementClicked(String surveyName) {
        surveyNameLabel.setText(surveyName);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_survey_setting, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    @Override
    public void onBackPressed(){
        // intentの作成
        Intent intent = new Intent();
        // intentへ添え字付で値を保持させる
        intent.putExtra("observer",this.userNameLabel.getText().toString());
        intent.putExtra(ToolsUtility.IntentKey_SurveyID,this.surveyNameLabel.getText().toString());
        // 返却したい結果ステータスをセットする
        setResult( Activity.RESULT_OK, intent );
        // アクティビティを終了させる
        this.finish();
    }

}
