package jp.ac.gifunct.ktajima.llservey;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.media.MediaPlayer;
import android.media.MediaRecorder;
import android.media.MediaScannerConnection;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.GridView;
import android.widget.Toast;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.concurrent.CountDownLatch;

public class TagsActivity extends AppCompatActivity {
    private Button addTagButton;
    private Button voiceRecordButton;
    private Button voicePlayButton;

    private GridView tagList;
    private ArrayAdapter<String> tagListAdapter;
    private HashMap<String,String> tagListMap = new HashMap<String,String>();
    private String targetID;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_tags);

        Intent intent = getIntent();
        if(intent != null){
            if(intent.hasExtra(ToolsUtility.IntentKey_PictureTargetID)) {
                this.targetID = intent.getStringExtra(ToolsUtility.IntentKey_PictureTargetID);
            }
        }

        this.addTagButton = (Button) findViewById(R.id.button_addtag);
        this.addTagButton.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                TagsActivity.this.addTagButtonClicked(v);
            }
        });
        this.voiceRecordButton = (Button) findViewById(R.id.button_voicerecord);
        this.voiceRecordButton.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                TagsActivity.this.voiceRecordButtonClicked(v);
            }
        });
        this.voicePlayButton = (Button) findViewById(R.id.button_voiceplay);
        this.voicePlayButton.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                TagsActivity.this.voicePlayButtonClicked(v);
            }
        });
        this.tagList = (GridView) findViewById(R.id.gridView_tag);
        this.tagListAdapter = new ArrayAdapter(this, R.layout.textlist);
        this.tagListAdapter.add("Key");
        this.tagListAdapter.add("Value");


        //TargetIDに従ってXMLをオープンしタグリストを埋める。
        this.tagListMap = ToolsUtility.openTagListFile(this.targetID);
        for(String key:this.tagListMap.keySet()){
            this.tagListAdapter.add(key);
            this.tagListAdapter.add(this.tagListMap.get(key));
        }
        this.tagList.setAdapter(tagListAdapter);
        //

        //degug
        //ToolsUtility.TagKeyList.add("sample tag 1");
        //ToolsUtility.TagKeyList.add("sample tag 2");
    }

    private MediaRecorder voiceRecorder;
    private MediaPlayer voicePlayer;
    boolean voiceRecording = false;
    boolean voicePlaying = false;
    private void voicePlayButtonClicked(View v) {
        //TODO 音声ファイル再生
        File voiceFile = ToolsUtility.getVoiceFile(this.targetID);
        if(!voiceFile.exists()){
            Toast ts = Toast.makeText(this, "No recorded file", Toast.LENGTH_LONG);
            ts.show();
        }
        if (!voicePlaying) {
            voicePlayer = new MediaPlayer();
            try {
                voicePlayer.setDataSource(voiceFile.getPath());
                voicePlayer.prepare();
                voicePlayer.start();
            } catch (IOException e){
                e.printStackTrace();
            }
            voicePlayButton.setText("STOP");
            voicePlaying = true;
            this.voiceRecordButton.setEnabled(false);
        } else {
            voicePlayer.stop();
            voicePlayer.release();
            voicePlayButton.setText("PLAY");
            voicePlaying = false;
            this.voiceRecordButton.setEnabled(true);
        }
    }

    private void voiceRecordButtonClicked(View v) {
        //音声ファイル録音
        File voiceFile = ToolsUtility.getVoiceFile(this.targetID);
        if (!voiceRecording) {
            voiceRecorder = new MediaRecorder();
            voiceRecorder.setOutputFile(voiceFile.getPath());
            // 入力ソースの指定
            voiceRecorder.setAudioSource(MediaRecorder.AudioSource.MIC);
            // フォーマット&コーデックの指定
            voiceRecorder.setOutputFormat(MediaRecorder.OutputFormat.THREE_GPP);
            voiceRecorder.setAudioEncoder(MediaRecorder.AudioEncoder.AMR_NB);
            try {
                // 録音準備
                voiceRecorder.prepare();
             } catch (IllegalStateException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            }
            // 録音開始
            voiceRecorder.start();
            voiceRecordButton.setText("STOP");
            voiceRecording = true;
            this.voicePlayButton.setEnabled(false);
        } else {
            // 録音停止
            try {
                voiceRecorder.stop();
                String[] paths = {voiceFile.getPath()};
                String[] mimeTypes = {"video/3gpp"};
                MediaScannerConnection.scanFile(getApplicationContext(),
                        paths,
                        mimeTypes,
                        null);
                voiceRecorder.reset();
            } catch (IllegalStateException e){
                e.printStackTrace();
            }
            voiceRecorder.release();
            Toast ts = Toast.makeText(this, "Recorded", Toast.LENGTH_LONG);
            ts.show();
            voiceRecordButton.setText("RECORD");
            voiceRecording = false;
            this.voicePlayButton.setEnabled(true);
        }
    }

    private String selectedTagKey;
    private String selectedTagValue;

    private void addTagButtonClicked(View v) {
        //キーの決定
        selectedTagKey = null;
        if (ToolsUtility.TagValueList.keySet().size() == 0) {
            openAddTextDialog("Add new Tag key",new DialogInterface.OnDismissListener() {
                @Override
                public void onDismiss(DialogInterface dialog) {
                    TagsActivity.this.tagKeySelected(addTextDialogString);
                }
            });
        } else {
            final String[] items = ToolsUtility.TagValueList.keySet().toArray(new String[0]);
            openChooseTagDialog("Select tag key", items, new DialogInterface.OnDismissListener() {
                @Override
                public void onDismiss(DialogInterface dialog) {
                    TagsActivity.this.tagKeySelected(chooseTextDialogString);
                }
            });
        }
    }

    /** key選択のダイアログが閉じた後に呼び出される */
    private void tagKeySelected(String selectedkey){
        selectedTagKey = selectedkey;
        if (selectedTagKey == null) {
            //Toast ts = Toast.makeText(this, "Key is null", Toast.LENGTH_LONG);
            //ts.show();
            return;
        } else {
            //新しい場合は追加
            if(!ToolsUtility.TagValueList.keySet().contains(selectedTagKey)) {
                ToolsUtility.TagValueList.put(selectedTagKey, new ArrayList<String>());
            }
            //debug
            //Toast ts = Toast.makeText(this, "Key is "+selectedTagKey, Toast.LENGTH_LONG);
            //ts.show();
            //
        }

        //値選択ダイアログを開く
        ArrayList<String> valueList = new ArrayList<String>();
        if(ToolsUtility.TagValueList.containsKey(selectedTagKey)) {
            valueList.addAll(ToolsUtility.TagValueList.get(selectedTagKey));
        }
        if (valueList.size() == 0) {
            openAddTextDialog("Add new tag value",new DialogInterface.OnDismissListener() {
                @Override
                public void onDismiss(DialogInterface dialog) {
                    TagsActivity.this.tagValueSelected(addTextDialogString);
                }
            });
        } else {
            final String[] items = valueList.toArray(new String[0]);
            openChooseValueDialog("Select tag value", items, new DialogInterface.OnDismissListener() {
                @Override
                public void onDismiss(DialogInterface dialog) {
                    TagsActivity.this.tagValueSelected(chooseTextDialogString);
                }
            });
        }

    }

    /** 値選択のダイアログが閉じた後に呼び出される */
    private void tagValueSelected(String selectedValue){
        selectedTagValue = selectedValue;
        if (selectedTagValue == null) {
            //Toast ts = Toast.makeText(this, "Key is null", Toast.LENGTH_LONG);
            //ts.show();
            return;
        } else {
            //新しい場合は追加
            ArrayList<String> valueList = new ArrayList<String>();
            if(ToolsUtility.TagValueList.containsKey(selectedTagKey)) {
                valueList.addAll(ToolsUtility.TagValueList.get(selectedTagKey));
            }
            if(!valueList.contains(selectedTagValue)) {
                valueList.add(selectedTagValue);
                ToolsUtility.TagValueList.put(selectedTagKey,valueList);
            }
            //debug
            //Toast ts = Toast.makeText(this, "Key is "+selectedTagKey, Toast.LENGTH_LONG);
            //ts.show();
            //
        }

        //表に値を追加
        this.tagListAdapter.add(this.selectedTagKey);
        this.tagListAdapter.add(this.selectedTagValue);
        this.tagListMap.put(this.selectedTagKey, this.selectedTagValue);
        File updateFile = ToolsUtility.saveTagListFile(this.targetID,this.tagListMap);
        if(updateFile != null) {
            String[] paths = {updateFile.getPath()};
            String[] mimeTypes = {"text/plain"};
            MediaScannerConnection.scanFile(getApplicationContext(),
                    paths,
                    mimeTypes,
                    null);
        }
    }

    private String chooseTextDialogString;
    private void openChooseTagDialog(String title, final String[] items,DialogInterface.OnDismissListener listner) {
        chooseTextDialogString = null;
        AlertDialog.Builder builder = new AlertDialog.Builder(this)
                .setTitle(title)
                .setItems(items, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        chooseTextDialogString = items[which];
                    }
                })
                .setPositiveButton("New", new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                                openAddTextDialog("Add new Tag key",new DialogInterface.OnDismissListener() {
                                    @Override
                                    public void onDismiss(DialogInterface dialog) {
                                        TagsActivity.this.tagKeySelected(addTextDialogString);
                                    }
                                });
                            }
                        }
                ).setNegativeButton("Cancel", null);
        Dialog dialog = builder.show();
        dialog.setOnDismissListener(listner);
    }
    private void openChooseValueDialog(String title, final String[] items,DialogInterface.OnDismissListener listner) {
        chooseTextDialogString = null;
        AlertDialog.Builder builder = new AlertDialog.Builder(this)
                .setTitle(title)
                .setItems(items, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        chooseTextDialogString = items[which];
                    }
                })
                .setPositiveButton("New", new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                                openAddTextDialog("Add new Tag Value",new DialogInterface.OnDismissListener() {
                                    @Override
                                    public void onDismiss(DialogInterface dialog) {
                                        TagsActivity.this.tagValueSelected(addTextDialogString);
                                    }
                                });
                            }
                        }
                ).setNegativeButton("Cancel", null);
        Dialog dialog = builder.show();
        dialog.setOnDismissListener(listner);
    }

    private String addTextDialogString;
    private void openAddTextDialog(String title,DialogInterface.OnDismissListener listner) {
        addTextDialogString = null;
        final EditText editView = new EditText(this);
        AlertDialog.Builder builder = new AlertDialog.Builder(this)
                .setIcon(android.R.drawable.ic_dialog_info)
                .setTitle(title)
                .setView(editView)
                .setPositiveButton("OK", new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int whichButton) {
                        addTextDialogString = editView.getText().toString();
                    }
                })
                .setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int whichButton) {
                    }
                });
        Dialog dialog = builder.show();

        dialog.setOnDismissListener(listner);
    }


    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_map, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    @Override
    public void onBackPressed(){
        // intentの作成
        Intent intent = new Intent();
        // 返却したい結果ステータスをセットする
        setResult( Activity.RESULT_OK, intent );
        // アクティビティを終了させる
        this.finish();
    }
}
