package jp.ac.gifunct.ktajima.llservey;

import android.content.Context;
import android.content.res.AssetManager;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Matrix;
import android.graphics.Typeface;
import android.media.Image;
import android.media.MediaScannerConnection;
import android.os.Environment;
import android.widget.GridLayout;

import org.xml.sax.SAXException;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.parsers.ParserConfigurationException;


/**
 * Created by ktajima on 2015/09/01.
 */
public class ToolsUtility {
    public static final String IntentKey_SurveyID = "surveyID";
    public static final String IntentKey_PictureTargetID = "tartgetID";

    public static HashMap<String,ArrayList<String>> TagValueList = new HashMap<String,ArrayList<String>>();
    public static ArrayList<DataListElement> DataListElements = new ArrayList<DataListElement>();
    public static File outputBaseDir = new File(Environment.getExternalStorageDirectory().getPath()+File.separator+"LLPictures");
    public static File outputDir;
    public static HashMap<String,SurveyData> dataSetting;

    static {
        dataSetting = getSettingData();
        outputBaseDir.mkdirs();
    }

    public static ArrayList<String> getTargetList(){
        ArrayList<String> list = new ArrayList<String>();
        for(DataListElement elem:DataListElements){
            if(!list.contains(elem.getSubID())){
                list.add(elem.getSubID());
            }
        }
        return list;
    }

    public static ArrayList<DataListElement> getTargetData(String targetID){
        ArrayList<DataListElement> list = new ArrayList<DataListElement>();
        for(DataListElement elem:DataListElements){
            if(elem.getSubID().equals(targetID)){
                list.add(elem);
            }
        }
        return list;
    }

    public static File makePictureFileName(String surveyName) {
        File FileDir = new File(outputBaseDir.getPath() + File.separator + surveyName);
        GregorianCalendar cal = new GregorianCalendar();
        SimpleDateFormat fmt = new SimpleDateFormat("yyyy-MM-dd_HH:mm:ss");
        File outputFile = new File(FileDir.getPath()+ File.separator + fmt.format(cal.getTime())+".jpg");
        return outputFile;
    }

    public static File makePictureFileName(String surveyName,String Filename) {
        File FileDir = new File(outputBaseDir.getPath() + File.separator + surveyName);
        File outputFile = new File(FileDir.getPath()+ File.separator + Filename);
        return outputFile;
    }

    /** 現在のデータをオープン */
    public static ArrayList<DataListElement> openDataList(String surveyName){
        ArrayList<DataListElement> elements = new ArrayList<DataListElement>();
        if(outputDir == null) return elements;
        File dataListFile = new File(outputDir.getPath()  + File.separator + "datalist.txt");
        try {
            if(dataListFile.exists()) {
                BufferedReader reader = new BufferedReader(new InputStreamReader(new FileInputStream(dataListFile),"UTF-8"));
                String Line;
                while ((Line = reader.readLine()) != null) {
                    DataListElement element = DataListElement.parseFromLine(Line);
                    if(element != null){
                        elements.add(element);
                    }
                }
            }
        }catch(IOException e){

        }
        return elements;
    }

    //撮影結果の追加
    public static void addDataListElements(ArrayList<DataListElement> list){
        if(DataListElements != null) {
            DataListElements.addAll(list);
        }
    }
    /** 現在のリストを保存 */
    public static File saveDataList(String surveyName,ArrayList<DataListElement> elements){
        if(outputDir == null) return null;
        File dataListFile = new File(outputDir.getPath() + File.separator + "datalist.txt");
        try {
            PrintWriter writer = new PrintWriter(new BufferedWriter(new OutputStreamWriter(new FileOutputStream(dataListFile),"UTF-8")));
            int count = 1;
            for(DataListElement data:elements){
                writer.println(data.toString(count++));
            }
            writer.close();
        }catch(IOException e){

        }
        return dataListFile;
    }

    public static void addNewSurvey(String surveyName){
        SurveyData data = new SurveyData(surveyName);
        dataSetting.put(surveyName, data);
        outputDir = new File(outputBaseDir.getPath() + File.separator + surveyName);
        outputDir.mkdirs();
        //XMLの更新
        saveSettingXML();
    }

    public static File saveSettingXML(){
        File dataListFile = new File(outputBaseDir.getPath() + File.separator + "setting.xml");
        try {
            PrintWriter writer = new PrintWriter(new BufferedWriter(new OutputStreamWriter(new FileOutputStream(dataListFile),"UTF-8")));
            writer.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
            writer.println("<data>");
            int count = 1;
            for(String surveyname:dataSetting.keySet()){
                SurveyData data = dataSetting.get(surveyname);
                writer.println("\t<survey>");
                writer.println("\t\t<name>"+surveyname+"</name>");
                writer.println("\t\t<title>"+data.getSurveyTitle()+"</title>");
                writer.println("\t\t<memo>"+data.getMemo()+"</memo>");
                //タグ関係
                HashMap<String,ArrayList<String>> tags = data.getTagElements();
                for(String key:tags.keySet()){
                    writer.println("\t\t<tag>");
                    writer.println("\t\t\t<key>"+key+"</key>");
                    ArrayList<String> values = tags.get(key);
                    for(String value:values){
                        writer.println("\t\t\t<value>"+value+"</value>");
                    }
                    writer.println("\t\t</tag>");
                }
                //
                writer.println("\t</survey>");
            }
            writer.println("</data>");
            writer.close();
        }catch(IOException e){

        }
        return dataListFile;
    }

    /** 調査設定を一式読み込む、なければ新規作成 */
    public static void startSurvey(String surveyName) {
        //保存用フォルダの作成
        outputDir = new File(outputBaseDir.getPath() + File.separator + surveyName);
        outputDir.mkdirs();
        //
        DataListElements = openDataList(surveyName);
        SurveyData sdata = dataSetting.get(surveyName);
        if(sdata == null){
            sdata = new SurveyData(surveyName);
        }
        TagValueList = sdata.getTagElements();
    }



    //XMLから調査一覧を取得
    public static HashMap<String,SurveyData> getSettingData(){
        File dataListFile = new File(outputBaseDir.getPath() + File.separator + "setting.xml");
        SettingXMLParser parser = new SettingXMLParser();
        try {
            parser.openXMLFile(dataListFile);
        } catch (IOException e) {
            e.printStackTrace();
        } catch (ParserConfigurationException e) {
            e.printStackTrace();
        } catch (SAXException e) {
            e.printStackTrace();
        }
        return parser.getDataMap();
    }

    public static HashMap<String,String> openTagListFile(String pictureTargetID){
        HashMap<String,String> table = new HashMap<String,String>();
        if(outputDir == null) return table;
        File dataListFile = new File(outputDir.getPath() + File.separator + pictureTargetID + ".txt");
        try {
            if(dataListFile.exists()) {
                BufferedReader reader = new BufferedReader(new InputStreamReader(new FileInputStream(dataListFile)));
                String Line;
                while ((Line = reader.readLine()) != null) {
                    String[] valuse = Line.split("\t");
                    if(valuse.length >= 2){
                        table.put(valuse[0],valuse[1]);
                    }
                }
            }
        }catch(IOException e){

        }
        return table;
    }

    public static File saveTagListFile(String myID, HashMap<String,String> table){
        if(outputDir == null) return null;
        File dataListFile = new File(outputDir.getPath() + File.separator + myID + ".txt");
        try {
            PrintWriter writer = new PrintWriter(new BufferedWriter(new OutputStreamWriter(new FileOutputStream(dataListFile))));
            for(String key:table.keySet()){
                writer.print(key);
                writer.print("\t");
                writer.print(table.get(key));
                writer.println();
            }
            writer.close();
        }catch(IOException e){

        }
        return dataListFile;
    }

    public static Typeface getTypefaceFromAssets( Context context, String path ) {
        return Typeface.createFromAsset(context.getAssets(), path);
    }

    public static Bitmap getNoPitureImage(Context context) throws IOException {
        AssetManager assetManager = context.getAssets();
        BufferedInputStream  bis = new BufferedInputStream(assetManager.open("nopicture.jpg"));
        return BitmapFactory.decodeStream(bis);
    }

    public static byte[] readFile(File file) throws IOException {
        ArrayList<Byte> dataList = new ArrayList<Byte>();
        BufferedInputStream reader = new BufferedInputStream(new FileInputStream(file));
        int i;
        while ((i = reader.read()) != -1) {
            dataList.add((byte) i);
        }

        byte[] data = new byte[dataList.size()];
        i = 0;
        for (Byte b : dataList) {
            data[i] = b;
            i++;
        }

        return data;
    }

    public static Bitmap openImageFromByteArray(byte[] array) throws IOException {
        Bitmap img = BitmapFactory.decodeByteArray(array, 0, array.length);
        return img;
    }

    public static Bitmap makeSquare(Bitmap image) {
        int w = image.getWidth(), h = image.getHeight();
        int center_x = w / 2, center_y = h / 2;

        Bitmap img = null;
        if (w < h) {
            img = Bitmap.createBitmap(image, 0, center_y - w / 2, w, w);
        } else {
            img = Bitmap.createBitmap(image, center_x - h / 2, 0, h, h);
        }


        return img;
    }

    public static Bitmap reduce1(Bitmap image, int dw, int dh) {
        Bitmap thumb = Bitmap.createScaledBitmap(image, dw, dh, false);
        return thumb;
    }

    public static Bitmap tilt(Bitmap image, double degree) {
        Matrix  matrix = new Matrix();
        matrix.postRotate((float) degree);
        Bitmap result = Bitmap.createBitmap(image, 0, 0, image.getWidth(), image.getHeight(), matrix, false);
        return result;
    }

    public static File getVoiceFile(String targetID) {
        return new File(outputDir + File.separator + targetID + ".3gp");
    }


    /**
    public static void main(String[] args) {
        File datafile = new File("C:\\Users\\t-koji\\Desktop\\2015-08-15 13.09.51.jpg");
        byte[] data = readFile(datafile);
        BufferedImage original;
        try {
            original = openImageFromByteArray(data);
            GraphicsConfiguration gc = getDefaultConfiguration();
            BufferedImage rotated1 = tilt(original, -Math.PI / 2, gc);
            BufferedImage rotated2 = tilt(original, +Math.PI / 4, gc);
            BufferedImage rotated3 = tilt(original, Math.PI, gc);
            display(original, rotated1, rotated2, rotated3);

        } catch (IOException ex) {
            Logger.getLogger(AndoroidKeikanToolFragment.class.getName()).log(Level.SEVERE, null, ex);
        }
     }
     */


}
