package jp.ac.gifunct.ktajima.llservey;

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Typeface;
import android.hardware.Sensor;
import android.location.Criteria;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.media.MediaScannerConnection;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.SeekBar;
import android.widget.TextView;
import android.widget.Toast;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;

public class shootingActivity extends AppCompatActivity implements LocationListener {
    private Button takephotoButton;
    private Button listButton;
    private Button mapButton;
    private Button tagsettingButton;
    private Button pictureNextButton;
    private Button picturePrevButton;
    private ImageView imageArea;
    private TextView pictureCounterLabel;
    private Button deleteButton;
    private TextView locationLabel;

    //位置情報取得関係
    private LocationManager locationManager;
    private String provider;
    private Location currentLocation;
    private Location picturedLocation;

    private int showPictureNumber = 0;
    private ArrayList<File> takepictures = new ArrayList<File>();

    public String surveyID;
    public String myTargetName;

    //
    public static final int CameraActivityCalled = 100;
    public static final int MapsActivityCalled = 200;
    public static final int TagActivityCalled = 400;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_shooting);


        // フォント読み込み
        Typeface typeface = ToolsUtility.getTypefaceFromAssets(this, "fontawesome-webfont.ttf");
        // コントロールの表示フォント指定
        this.takephotoButton = (Button) findViewById(R.id.takephoto);
        this.takephotoButton.setTypeface(typeface);
        this.takephotoButton.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                shootingActivity.this.PhotoButtonClicked(v);
            }
        });

        this.listButton = (Button) findViewById(R.id.button_list);
        this.listButton.setTypeface(typeface);
        this.listButton.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                shootingActivity.this.nextButtonClicked(v);
            }
        });

        this.mapButton = (Button) findViewById(R.id.button_map);
        this.mapButton.setTypeface(typeface);
        this.mapButton.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                shootingActivity.this.mapOpenButtonClicked(v);
            }
        });
        this.tagsettingButton = (Button) findViewById(R.id.button_tags);
        this.tagsettingButton.setTypeface(typeface);
        this.tagsettingButton.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                shootingActivity.this.tagsettingButtonClicked(v);
            }
        });

        this.imageArea = (ImageView) findViewById(R.id.imageView);

        this.pictureNextButton = (Button) findViewById(R.id.button_next);
        this.pictureNextButton.setEnabled(false);
        this.pictureNextButton.setTypeface(typeface);
        this.pictureNextButton.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                shootingActivity.this.pictureNextButtonClicked(v);
            }
        });

        this.picturePrevButton = (Button) findViewById(R.id.button_prev);
        this.picturePrevButton.setTypeface(typeface);
        this.picturePrevButton.setEnabled(false);
        this.picturePrevButton.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                shootingActivity.this.picturePrevButtonClicked(v);
            }
        });

        this.deleteButton = (Button) findViewById(R.id.button_delete);
        //this.picturePrevButton.setTypeface(typeface);
        this.deleteButton.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                shootingActivity.this.deleteButtonClicked(v);
            }
        });

        this.pictureCounterLabel = (TextView) findViewById(R.id.piccount);

        //既存のリスト読み込み化の判定
        Intent intent = getIntent();
        if(intent != null){
            if(intent.hasExtra(ToolsUtility.IntentKey_SurveyID)) {
                this.surveyID = intent.getStringExtra(ToolsUtility.IntentKey_SurveyID);
            }
            if(intent.hasExtra(ToolsUtility.IntentKey_PictureTargetID)) {
                this.myTargetName = intent.getStringExtra(ToolsUtility.IntentKey_PictureTargetID);
            }        }
        if(this.myTargetName != null){
            //既存データが存在する場合(myTargetNameはセット済み)

            //写真リストを読み込む
            ArrayList<DataListElement> elementList = ToolsUtility.getTargetData(this.myTargetName);
            if(elementList.size() != 0) {
                for (DataListElement elem : elementList) {
                    File pictureFile = ToolsUtility.makePictureFileName(this.surveyID, elem.getFileName());
                    this.takepictures.add(pictureFile);
                }
                this.showPictureNumber = 1;
                this.setCountBar();
                File picture = this.takepictures.get(this.showPictureNumber - 1);
                Bitmap bitmap = BitmapFactory.decodeFile(picture.getPath());
                this.imageArea.setImageBitmap(bitmap);
                this.deleteButton.setEnabled(true);
            } else {
                //画像なしの表示
                try {
                    Bitmap bitmap = ToolsUtility.getNoPitureImage(this);
                    this.imageArea.setImageBitmap(bitmap);
                } catch (IOException e) {

                }
                this.deleteButton.setEnabled(false);
            }

        } else {
            //既存データがない場合
            //対象番号の追加
            String myID = new SimpleDateFormat("yyyyMMddHHmmss").format(new Date());
            if(ToolsUtility.getTargetList().contains(myID)){
                myID = myID + "'";
            }
            this.myTargetName = myID;

            //画像なしの表示
            try {
                Bitmap bitmap = ToolsUtility.getNoPitureImage(this);
                this.imageArea.setImageBitmap(bitmap);
            } catch (IOException e) {

            }
            this.deleteButton.setEnabled(false);
        }




        //位置情報関係
        locationManager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);
        Criteria criteria = new Criteria();
        provider = locationManager.getBestProvider(criteria, false);
        //Location location = this.locationManager.getLastKnownLocation(provider);
        this.locationLabel = (TextView)findViewById(R.id.textview_location);

    }



    private void tagsettingButtonClicked(View v) {
        Intent intent = new Intent(shootingActivity.this, TagsActivity.class);
        intent.putExtra(ToolsUtility.IntentKey_PictureTargetID,this.myTargetName);
        startActivityForResult(intent, TagActivityCalled);
    }

    @Override
    protected void onResume() {
        super.onResume();
        locationManager.requestLocationUpdates(provider, 400, 1, this);
    }

    @Override
    protected void onPause() {
        super.onPause();
        locationManager.removeUpdates(this);
    }

    private void mapOpenButtonClicked(View v) {
        Intent intent = new Intent(shootingActivity.this, MapsActivity.class);
        double lat = 0, lng = 0, alt = 0;
        if(this.picturedLocation != null) {
            lat = this.picturedLocation.getLatitude();
            lng = this.picturedLocation.getLongitude();
            alt = this.picturedLocation.getAltitude();
            intent.putExtra("place","Pictured");
        } else if(this.currentLocation != null) {
            lat = this.currentLocation.getLatitude();
            lng = this.currentLocation.getLongitude();
            alt = this.currentLocation.getAltitude();
            intent.putExtra("place","Current");
        }
        intent.putExtra("lat",lat);
        intent.putExtra("long",lng);
        intent.putExtra("alt", alt);
        startActivityForResult(intent, MapsActivityCalled);
    }

    private void deleteButtonClicked(View v) {
        if(this.takepictures.size() <= 0){
            return;
        }

        File deleteFile = null;
        if(this.showPictureNumber > 0){
            deleteFile = this.takepictures.get(this.showPictureNumber-1);
            this.takepictures.remove(this.showPictureNumber-1);
        }

        if(this.takepictures.size() == 0) {
            try {
                Bitmap bitmap = ToolsUtility.getNoPitureImage(this);
                this.imageArea.setImageBitmap(bitmap);
            } catch (IOException e) {
            }
        } else {
            if(this.showPictureNumber > 1){
                this.showPictureNumber--;
            }
            File picture = this.takepictures.get(this.showPictureNumber-1);
            Bitmap bitmap = BitmapFactory.decodeFile(picture.getPath());
            this.imageArea.setImageBitmap(bitmap);
        }

        this.setCountBar();
        /** ファイルの削除 */
        if(deleteFile != null){
            deleteFile.delete();
        }
    }

    @Override
    public void onLocationChanged(Location location) {
        this.currentLocation = location;
        this.locationLabel.setText("Location: OK ("+location.getAccuracy()+" m)");
    }

    @Override
    public void onStatusChanged(String provider, int status, Bundle extras) {
    }

    @Override
    public void onProviderEnabled(String provider) {
    }

    @Override
    public void onProviderDisabled(String provider) {
    }

    private void PhotoButtonClicked(View v) {
        Intent intent = new Intent(shootingActivity.this, CameraActivity.class);
        intent.putExtra(ToolsUtility.IntentKey_SurveyID,this.surveyID);
        intent.putExtra(ToolsUtility.IntentKey_PictureTargetID,this.myTargetName);
        startActivityForResult(intent, CameraActivityCalled);
    }
    private void pictureNextButtonClicked(View v) {
        if(this.showPictureNumber < this.takepictures.size()){
            File picture = this.takepictures.get(this.showPictureNumber);
            this.showPictureNumber++;
            Bitmap bitmap = BitmapFactory.decodeFile(picture.getPath());
            this.imageArea.setImageBitmap(bitmap);
        }
        this.picturePrevButton.setEnabled(true);
        this.setCountBar();
    }
    private void picturePrevButtonClicked(View v) {
        if(this.showPictureNumber > 1){
            File picture = this.takepictures.get(this.showPictureNumber - 2);
            this.showPictureNumber--;
            Bitmap bitmap = BitmapFactory.decodeFile(picture.getPath());
            this.imageArea.setImageBitmap(bitmap);
        }
        this.setCountBar();
    }
    /** 写真撮影枚数の表示関係のリフレッシュ */
    private void setCountBar(){
        SeekBar bar = (SeekBar)findViewById(R.id.seekBar);
        bar.setMax(this.takepictures.size());
        bar.setProgress(this.showPictureNumber);

        if(this.takepictures.size() == 0 ) { this.showPictureNumber = 0;}
        this.pictureCounterLabel.setText(this.showPictureNumber + "/" + this.takepictures.size());
        if(this.showPictureNumber >= this.takepictures.size()){
            this.pictureNextButton.setEnabled(false);
        } else {
            this.pictureNextButton.setEnabled(true);
        }
        if(this.showPictureNumber <= 1){
            this.picturePrevButton.setEnabled(false);
        } else {
            this.picturePrevButton.setEnabled(true);
        }
        if(this.takepictures.size() > 0){
            this.deleteButton.setEnabled(true);
        } else {
            this.deleteButton.setEnabled(false);
        }

    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_shooting, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if( resultCode == Activity.RESULT_OK ){
            if(requestCode == CameraActivityCalled) {
                // 返却されてきたintentから値を取り出す
                File lastpic = new File(data.getStringExtra("lastpic"));
                if(lastpic != null){
                    this.takepictures.add(lastpic);
                    this.pictureCounterLabel.setText(this.takepictures.size()+"/"+this.takepictures.size());
                    this.showPictureNumber = this.takepictures.size();
                    if(this.takepictures.size() > 1) {
                        this.picturePrevButton.setEnabled(true);
                    }
                    this.setCountBar();
                    Bitmap bitmap = BitmapFactory.decodeFile(lastpic.getPath());
                    this.imageArea.setImageBitmap(bitmap);

                    //撮影位置を記録する。
                    if(this.currentLocation != null) {
                        this.picturedLocation = this.currentLocation;
                    }
                }
            } else if (requestCode == MapsActivityCalled){
                //マップ画面が呼ばれた場合は変更された位置情報を記録する。
                double Latitude = data.getDoubleExtra("lat",0);
                double Longitude = data.getDoubleExtra("lon",0);
                if(this.picturedLocation == null){
                    this.picturedLocation = new Location("input");
                }
                this.picturedLocation.setLongitude(Longitude);
                this.picturedLocation.setLatitude(Latitude);
            } else if (requestCode == TagActivityCalled){
                //タグ設定画面が呼ばれた場合は、XML設定ファイルの調査データ、タグ情報を更新する。
                HashMap<String,ArrayList<String>> tagList = ToolsUtility.TagValueList;
                SurveyData sdata = ToolsUtility.dataSetting.get(this.surveyID);
                if(sdata == null){
                    sdata = new SurveyData(this.surveyID);
                }
                sdata.setTagList(tagList);
                ToolsUtility.dataSetting.put(this.surveyID,sdata);
                File updateFile = ToolsUtility.saveSettingXML();
                String[] paths = {updateFile.getPath()};
                String[] mimeTypes = {"application/xml"};
                MediaScannerConnection.scanFile(getApplicationContext(),
                        paths,
                        mimeTypes,
                        null);

            }



        }
        //super.onActivityResult(requestCode, resultCode, data);
    }



    boolean nextPicture = false;
    private void nextButtonClicked(View v) {
        this.nextPicture = true;
        // アクティビティを終了させる
        this.onBackPressed();
    }

    @Override
    public void onBackPressed(){
        //現在の状況を保存する。
        ArrayList<DataListElement> elements = new ArrayList<DataListElement>();
        for(File pictureFile:this.takepictures){
            if(this.picturedLocation == null){
                this.picturedLocation = new Location("none");
                this.picturedLocation.setLatitude(0);
                this.picturedLocation.setLongitude(0);
                this.picturedLocation.setAltitude(0);
            }
            DataListElement elem = new DataListElement(this.surveyID,pictureFile.getName(),new Date(pictureFile.lastModified()),this.picturedLocation,this.myTargetName);
            elements.add(elem);
        }
        ToolsUtility.addDataListElements(elements);
        File savedCSV = ToolsUtility.saveDataList(this.surveyID,ToolsUtility.DataListElements);
        if(savedCSV != null){
            String[] paths = {savedCSV.getPath()};
            String[] mimeTypes = {"text/plain"};
            MediaScannerConnection.scanFile(getApplicationContext(),
                    paths,
                    mimeTypes,
                    null);
        }

        // intentの作成
        Intent intent = new Intent();
        // intentへ添え字付で値を保持させる
        //intent.putExtra("lat",this.savePoint.latitude);
        //intent.putExtra("lon",this.savePoint.longitude);
        // 返却したい結果ステータスをセットする
        if(this.nextPicture == true){
            intent.putExtra("continue", "true");
        }
        setResult(Activity.RESULT_OK, intent);
        // アクティビティを終了させる
        this.finish();
    }
}
